#  High-Resolution Melting Background Removal (MeltBaseline)

##############################################################
# MIT License

# Copyright (c) [2020] [ZACHARY L. DWIGHT]

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
##############################################################

# Important Note: MELTING CURVE ANALYSIS Data Generation IP

# The generation of actual melt data may be covered by 
# U.S. Patent Nos. 7,582,429; 7,803,551; 8,068,992; 8,093,002; 
# 8,296,074; 9,273,346; and other U.S. and foreign patents and 
# patent applications owned by the University of Utah Research 
# Foundation and licensed to BioFire Defense, LLC. 

# If your PCR instrument is not licensed under these patents, 
# please contact Jill Powlick at bioMerieux (jill.powlick@biomerieux.com) 
# for sublicensing information.



#INPUTS DEFINED

#t is the array of temperatures
#h is the array of raw fluorescence values
#c is the width of the cursors (areas before and after melt occurs)

#OUTPUT DEFINED
# returns new array of fluorescence values, user may want to normalize to 100%
# as opposed to 1 (max helicity) and 0 (no helicity)

module MELTBASELINE
class MeltCurve
  def self.normalize(hs,ts,cw)
    # Select a number of points that represents the region BEFORE melting
    tcstart = 0
    tcend = cw
    
    # Select a number of points that represents the region AFTER melting
    bcstart = hs.length() - cw
    bcend = hs.length() - 1
    
    #find slope of cursor regions
    m1 = self.getslope(ts[tcstart],ts[tcend],hs[tcstart],hs[tcend])
    m2 = self.getslope(ts[bcstart],ts[bcend],hs[bcstart],hs[bcend])
    
    #define intercepts
    b1 = self.getintercept(ts[tcend],hs[tcend],m1)
    b2 = self.getintercept(ts[bcstart],hs[bcstart],m2)
    newF = 1.000
    a = []
    for i in (0..hs.length()-1)
		topF=self.getpoint(m1,ts[i],b1);
		
		botF=self.getpoint(m2,ts[i],b2);
		
		newF = (hs[i].to_f - botF.to_f )/(topF.to_f - botF.to_f );
		if(newF < 0 )
			newF = 0
		end
		if(newF > 1)
			newF = 1
		end
		a.push(newF);	
	end
    return a
  end
  
  def self.getintercept(x,y,m)
	b = y-(x*m)
	return b
  end
  
  def self.getpoint(m,x,b)
	point = m*x+b
	return point
  end
  
  def self.getslope(t1,t2,f1,f2)
	slope = (f2-f1)/(t2-t1)
	return slope
  end

end
end

##### EXAMPLE USAGE #####
# require 'melt_baseline'

# t = [60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95]
# h = [61572,60624,59537,58351,56644,54569,53361,51715,49491,48180,46515,45691,45097,44579,44050,44440,43907,39960,33507,26271,18696,11805,6121,2377,1278,2492,3287,3259,3184,3180,3094,2993,2880,2683,2508,2342]
# c = 6

# puts MELTBASELINE::MeltCurve.normalize(h,t,c)
